Package: rdmulti
Type: Package
Title: Robust Local Polynomial Methods for RD Designs with Multiple
        Cutoffs or Multiple Scores
Version: 2.0.0
Date: 2026-05-15
Authors@R: c(person(given = c("Matias", "D."),
                      family = "Cattaneo",
                      role = c("aut", "cre"),
                      email = "matias.d.cattaneo@gmail.com"),
               person(given = "Rocio",
                      family = "Titiunik",
                      role = "aut",
                      email = "rocio.titiunik@gmail.com"),
               person(given = "Gonzalo",
                      family = "Vazquez-Bare",
                      role = "aut",
                      email = "gvazquezbare@gmail.com"))
Description: The 'rdmulti' package implements estimation, inference, and graphical procedures for regression discontinuity (RD) designs with multiple cutoffs or multiple scores. rdmc() provides point estimation and robust bias-corrected inference for multi-cutoff designs, rdmcplot() provides data-driven RD plots for multi-cutoff designs, and rdms() provides point estimation and robust bias-corrected inference for multi-score designs. See Cattaneo, Titiunik and Vazquez-Bare (2020) <https://rdpackages.github.io/references/Cattaneo-Titiunik-VazquezBare_2020_Stata.pdf> for further methodological details.
Imports: ggplot2, rdrobust, rlang
License: GPL-3
Encoding: UTF-8
URL: https://github.com/rdpackages/rdmulti
BugReports: https://github.com/rdpackages/rdmulti/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-16 21:40:59 UTC; CodexSandboxOffline
Author: Matias D. Cattaneo [aut, cre],
  Rocio Titiunik [aut],
  Gonzalo Vazquez-Bare [aut]
Maintainer: Matias D. Cattaneo <matias.d.cattaneo@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-17 22:40:07 UTC
Built: R 4.6.0; ; 2026-05-18 00:56:37 UTC; unix
