# Generated by roxygen2: do not edit by hand

export(rdmc)
export(rdmcplot)
export(rdms)
import(ggplot2)
import(rdrobust)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,barplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(rlang,.data)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,poly)
importFrom(stats,qnorm)
